#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 16, 2); // Set the LCD address to 0x27 for a 16 chars and 2 line display

#include <SoftwareSerial.h> 
#define DF_rxPin 2 // DFplayer RX -> arduino 2
#define DF_txPin 3 // DFplayer TX -> arduino 3 
SoftwareSerial dfSerial(DF_txPin, DF_rxPin); // (RX, TX)

// DFplayer command 변수
uint8_t df_command[] = {0x7E, 0xFF, 0x06, 0x00, 0x00, 0x00, 0x00, 0xEF};

void send_command(uint8_t cmd, uint8_t ack, uint8_t msb, uint8_t lsb) {
  df_command[3] = cmd;
  df_command[4] = ack;
  df_command[5] = msb;
  df_command[6] = lsb;
  for(int i = 0; i < sizeof(df_command); i++){
    dfSerial.write(df_command[i]);
  }
}

int h = 12; // initial Time display is 12:59:45 PM
int m = 59;
uint8_t s = 45;
bool meridian = true;
uint8_t hm;
bool now_am = true; //AM
uint16_t yy = 2019;
uint8_t mm = 1;
uint8_t dd = 1;
uint8_t week_num = 1;
String week_day = "SUN";
bool adjust = false;

bool display_t = false;
unsigned long int start_time = 0;
uint8_t volume = 10;

bool receive_status = false; // 질의 코드 0x42수신 여부 플래그
uint8_t status_count = 0;   // 질의값 저장 인덱스용 카운터
uint8_t status_val[7];         // 질의값 저장 배열
bool played = false;         // 질의결과 플래그 (재생중인지 아닌지)
bool status_check = false; // 질의 수신 완료 플래그

bool speak = false; // 현재 상태 질의 시작 및 음성출력 시작 플래그

unsigned long int count_time = 0; 
bool delay_set = false; 
uint8_t sequence = 0;

void setup() {
  Serial.begin(9600);
  dfSerial.begin(9600); // DFplayer Serial
  send_command(0x06, 0, 0, volume); // 볼륨 10
  lcd.begin();
  lcd.backlight();
  lcd.clear();
  lcd.setCursor(0,0);  // row, col의 좌표로 커서를 위치 col: 0 ~ 15 row: 0, 1
  lcd.print("Arduino clock");
}

void loop() {
  cal_time();
  adjust_time();
  delayCount();
  speak_time();
  if (dfSerial.available()) {   // 42 02 01 
    uint8_t temp = dfSerial.read(); 
    if (temp == 0x42) receive_status = true;  
    if (receive_status == true) { 
      status_val[status_count] = temp; 
      status_count++;  
      if (temp == 0xEF) { // 질의값 수신종료 
        status_check = true; 
        receive_status = false;   
        status_count = 0;  
        if (status_val[2] == 2 && status_val[3] == 1) played = true; // 인덱스 2 = 2 -> SD카드, 인덱스 3 = 1 재생중
        else if (status_val[2] == 2 && status_val[3] == 0) played = false; // 인덱스 3 = 0 정지중
      } 
    } 
  }
}

void adjust_time() {
  if (Serial.available() > 0) {
    String temp = Serial.readStringUntil('\n');
    if (temp == "dtime") display_time();
    else if (temp == "s") { s = 0; start_time = millis(); display_t = true; }
    else if (temp == "mm") { m++; display_t = true; }
    else if (temp == "m") { m--; display_t = true; }
    else if (temp == "hh") { h++; display_t = true; }
    else if (temp == "h") { h--; display_t = true; }
    else if (temp == "24") { meridian = !meridian; display_t = true; }
    else if (temp == "yy") { yy++; display_t = true; }
    else if (temp == "y") { yy--; display_t = true; }
    else if (temp == "mon") { mm++; adjust = true; display_t = true; }
    else if (temp == "mon-") { mm--; adjust = true; display_t = true; }
    else if (temp == "dd") { dd++; adjust = true; display_t = true; }
    else if (temp == "d") { dd--; adjust = true; display_t = true; }
    else if (temp == "ww") { week_num++; adjust = true; display_t = true; }
    else if (temp == "w") { week_num--; adjust = true; display_t = true; }
    else if (temp.startsWith("tset")) {
      String set = temp.substring(4, 6);
      h = set.toInt();
      set = temp.substring(6, 8);
      m = set.toInt();
      set = temp.substring(8, 10);
      s = set.toInt();
      start_time = millis();
      adjust = true;
      display_t = true;
    }
    else if (temp.startsWith("dset")) {
      String set = temp.substring(4, 8);
      yy = set.toInt();
      set = temp.substring(8, 10);
      mm = set.toInt();
      set = temp.substring(10, 12);
      dd = set.toInt();
      adjust = true;
      display_t = true;
    }
    else if (temp == "test") speak = true;
    else if (temp == "stop") send_command(0x16, 0, 0, 0); 
    else if (temp == "play") send_command(0x0F, 0, 2, 6);
    else if (temp == "vv") { if(volume < 30) volume++; send_command(0x06, 0, 0, volume); }
    else if (temp == "v") { if(volume > 0) volume--; send_command(0x06, 0, 0, volume); }
    else if (temp == "pause") send_command(0x0E, 1, 0, 0);
    else if (temp == "previous") send_command(0x02, 1, 0, 0);
    else if (temp == "next") send_command(0x01, 1, 0, 0);
    else if (temp == "reset") send_command(0x0C, 0, 0, 0);
    if (m >= 60) m = 0;
    else if (m < 0) m = 59;
    if (h >= 24) h = 0; 
    else if (h < 0) h = 23; 
    if (h < 12) now_am = true;
    else now_am = false; 
    hm = h; 
    if (meridian == true && hm == 12) { now_am = false; }
    else if (meridian == true && hm > 12) { hm = hm - 12; }
    cal_dd();
    if (display_t == true) { display_time(); display_t = false; }
  }
}

void delayCount() { // 시간값을 갖는 카운터 함수
  if (delay_set == true) { 
    if (millis() - count_time >= 50) { // 시간 간격: 밀리초 
      count_time = millis(); 
      sequence++; // 시간값을 갖는 카운터
      if (sequence == 6) speak = true; // 실행코드를 한번만 실행토록 하는 조건
      else if (sequence == 23) speak = true; 
      else if (sequence == 46) speak = true; 
      else if (sequence == 67) speak = true; 
      else if (sequence == 101) speak = true; 
    } 
  } 
}

void speak_time() { 
  if (speak == true) { 
    if (sequence == 0 && status_check == false) { 
      send_command(0x42, 0, 0, 0); 
      sequence++; 
    } 
    if (status_check == true) { 
      if (sequence == 1) { 
        send_command(0x06, 0, 0, 30); // 볼륨 30 
        sequence++; 
      } 
      else if (sequence == 2 && played == true) { 
        sequence = 5; 
        delay_set = true; 
      } 
      else if (sequence == 2 && played == false) { 
        send_command(0x0F, 0, 2, 255); 
        sequence++; 
      } 
      else if (sequence == 3 && played == false) { 
        delay_set = true; 
        sequence++; 
      } 
      else if (sequence == 6) { 
        int ad_num = 1302;  
        uint8_t ad_lsb = ad_num; 
        uint8_t ad_msb = ad_num >> 8; 
        send_command(0x13, 0, ad_msb, ad_lsb); 
        speak = false; 
      } 
      else if (sequence == 23) { 
        int ad_num = 1005;  
        uint8_t ad_lsb = ad_num; 
        uint8_t ad_msb = ad_num >> 8; 
        send_command(0x13, 0, ad_msb, ad_lsb); 
        speak = false; 
      } 
      else if (sequence == 46) { 
        int ad_num = 1130;  
        uint8_t ad_lsb = ad_num; 
        uint8_t ad_msb = ad_num >> 8; 
        send_command(0x13, 0, ad_msb, ad_lsb); 
        speak = false; 
      } 
      else if (sequence == 67) { 
        int ad_num = 1203;  
        uint8_t ad_lsb = ad_num; 
        uint8_t ad_msb = ad_num >> 8; 
        send_command(0x13, 0, ad_msb, ad_lsb); 
        speak = false; 
      } 
      else if (sequence == 101) { 
        Serial.println("enter5"); 
        send_command(0x06, 0, 0, volume); 
        speak = false; 
        status_check = false; 
        delay_set = false; 
        sequence = 0; 
      } 
    } 
  } 
}

bool change_millis = false; 
uint16_t millis_sec = 994;

void cal_time() {
  if (millis() - start_time >= millis_sec) { // 시간 간격: 밀리초
    start_time = millis(); // 상기 조건을 만족할때의 밀리초를 다시 start_time에 저장하여 조건 초기화
    change_millis = ! change_millis;
    if (change_millis == true)  millis_sec = 995;
    else millis_sec = 994;
    s++;
    if (s == 60) {
      s = 0;
      m++;
    }
    if(m == 60) { m = 0; h++; }
    if(h == 12) { now_am = false;}
    else if (h == 24) { h = 0; now_am = true; dd++; week_num++; cal_dd(); } 
    hm = h;
    if(meridian == true && hm == 12) { now_am = false; }
    else if (meridian == true && hm > 12) { hm = hm - 12; }
    else now_am = true;
    display_time();
    if (h == 1 && m == 1 && s == 10) { // NTP 동기화 시간
    }
  }
}

void cal_dd() {
  uint8_t m_dds;
  switch (mm) {
    case 1:   m_dds = 31; break;
    case 2:   m_dds = 28; break;
    case 3:   m_dds = 31; break;
    case 4:   m_dds = 30; break;
    case 5:   m_dds = 31; break;
    case 6:   m_dds = 30; break;
    case 7:   m_dds = 31; break;
    case 8:   m_dds = 31; break;
    case 9:   m_dds = 30; break;
    case 10:  m_dds = 31; break;
    case 11:  m_dds = 30; break;
    case 12:  m_dds = 31; break;
  }
  if (adjust == false) {
    if (dd > m_dds) { dd = 1; mm++; }
    if (mm > 12) { mm = 1; yy++; }
    if (week_num > 7) week_num = 1;
    week_day_converter();
  }
  else {
    if (dd > m_dds) dd = 1;
    else if (dd < 1) dd = m_dds;
    if (mm > 12) mm = 1;
    else if (mm < 1) mm = 12;
    if (week_num > 7) week_num = 1;
    else if (week_num < 1) week_num = 7; 
    week_day_converter();
    adjust = false;
  }
}

void week_day_converter() {
  switch (week_num) {
    case 1:   week_day = "SUN"; break;
    case 2:   week_day = "MON"; break;
    case 3:   week_day = "TUE"; break;
    case 4:   week_day = "WED"; break;
    case 5:   week_day = "THU"; break;
    case 6:   week_day = "FRI"; break;
    case 7:   week_day = "SAT"; break;
  }
}

void display_time() {
  Serial.print(yy); Serial.print("."); 
  if (mm < 10) { Serial.print("0"); Serial.print(mm); Serial.print("."); }
  else { Serial.print(mm); Serial.print("."); }
  if (dd < 10) { Serial.print("0"); Serial.print(dd); Serial.print(". "); }
  else { Serial.print(dd); Serial.print(". "); }
  Serial.print(week_day); Serial.print(" ");  
  if (meridian == true) {
    if (now_am == true) Serial.print("AM ");
    else Serial.print("PM ");
    if (hm < 10) { Serial.print("0"); Serial.print(hm); Serial.print(":"); }
    else { Serial.print(hm); Serial.print(":"); }
  }
  else {
    if (h < 10) { Serial.print("0"); Serial.print(h); Serial.print(":"); }
    else { Serial.print(h); Serial.print(":"); }
  }
  if (m < 10) { Serial.print("0"); Serial.print(m); Serial.print(":"); }
  else { Serial.print(m); Serial.print(":"); }
  if (s < 10) { Serial.print("0"); Serial.println(s); }
  else Serial.println(s);
  lcd.setCursor(0,0); 
  lcd.print(yy); lcd.print("."); 
  if (mm < 10) { lcd.print("0"); lcd.print(mm); lcd.print("."); }
  else { lcd.print(mm); lcd.print("."); }
  if (dd < 10) { lcd.print("0"); lcd.print(dd); lcd.print(". "); }
  else { lcd.print(dd); lcd.print(". "); }
  lcd.print(week_day); lcd.print(" ");  
  lcd.setCursor(0,1);
  if (meridian == true) {
    if (now_am == true) lcd.print("AM ");
    else lcd.print("PM ");
    if (hm < 10) { lcd.print("0"); lcd.print(hm); lcd.print(":"); }
    else { lcd.print(hm); lcd.print(":"); }
  }
  else {
    if (h < 10) { lcd.print("0"); lcd.print(h); lcd.print(":"); }
    else { lcd.print(h); lcd.print(":"); }
  }
  if (m < 10) { lcd.print("0"); lcd.print(m); lcd.print(":"); }
  else { lcd.print(m); lcd.print(":"); }
  if (s < 10) { lcd.print("0"); lcd.print(s); lcd.print("     "); }
  else { lcd.print(s); lcd.print("     "); }  // 크리스탈 LCD에는 pritln 명령어 없음
}
